//==============================================================
// 萔
float4x4 World;
float4x4 WView;
float4x4 WVProj;
float4 Diffuse;
float4 Ambient;
float3 Light;
float3 LightVPos;
float3 LightColor;
texture MeshTex;
texture InputTex;
texture BloomTex;
float2 SampleOffset[16];
float SampleWeight[16];
float MinBright;
float ToneScale;
float IntScale;

//==============================================================
// eNX`Tv
sampler MeshSmp=sampler_state {
	Texture=<MeshTex>;
	MipFilter=LINEAR;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
};
sampler InputSmp=sampler_state {
	Texture=<InputTex>;
	MipFilter=LINEAR;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
	AddressU=BORDER;
	AddressV=BORDER;
};
sampler BloomSmp=sampler_state {
	Texture=<BloomTex>;
	MipFilter=LINEAR;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
	AddressU=BORDER;
	AddressV=BORDER;
};

//==============================================================
// VF[_iHDRV[j

// _VF[_
void HDRSceneVS(
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0,
	out float4 Pos : POSITION,
	out float2 MeshUV : TEXCOORD0,
	out float3 VPos : TEXCOORD1, 
	out float3 VNormal : TEXCOORD2
) {
	// _WAeNX`W
	Pos=mul(pos, WVProj);
	MeshUV=mesh_uv;

	// _W̃r[ϊ
	VPos=mul(pos, WView);
	VNormal=normalize(mul(normal, (float3x3)WView));
}

// sNZVF[_
float4 HDRScenePS(
	float2 MeshUV : TEXCOORD0,
	float3 VPos : TEXCOORD1, 
	float3 VNormal : TEXCOORD2
) : COLOR {
	
	// _烉Cgւ̃xNg
	float3 lp=LightVPos-VPos;

	// @ƃCg̊pxA_ƃCg̋疾邳vZ
	float3 color=Ambient+
		max(0, dot(normalize(lp), VNormal))*LightColor
		/(dot(lp, lp)+1);
	
	// 邳eNX`FAfBt[YFƍ
	return float4(tex2D(MeshSmp, MeshUV)*color*Diffuse*IntScale, 1);
}

//==============================================================
// VF[_iPx̒oj
float4 BrightPS(
	float2 UV : TEXCOORD0
) : COLOR {
	return float4(max(0, tex2D(InputSmp, UV)/IntScale-MinBright).rgb, 1);
}

//==============================================================
// VF[_iu[̍쐬j
float4 BloomPS(
	float2 UV : TEXCOORD0
) : COLOR {
	float3 color=0;
	for (int i=0; i<15; i++) {
		color+=tex2D(InputSmp, UV+SampleOffset[i])*SampleWeight[i];
	}
	return float4(color, 1);
}

//==============================================================
// VF[_iŏIój
float4 FinishPS(
	float2 UV : TEXCOORD0
) : COLOR {
	float3 color=
		tex2D(InputSmp, UV)*ToneScale/IntScale+tex2D(BloomSmp, UV);
	return float4(color, 1);
}

//==============================================================
// eNjbN
technique HDRScene {
	pass P0 {
		VertexShader=compile vs_1_1 HDRSceneVS();
		PixelShader=compile ps_2_0 HDRScenePS();
	}
}
technique Bright {
	pass P0 {
		PixelShader=compile ps_2_0 BrightPS();
	}
}
technique Bloom {
	pass P0 {
		PixelShader=compile ps_2_0 BloomPS();
	}
}
technique Finish {
	pass P0 {
		PixelShader=compile ps_2_0 FinishPS();
	}
}

