#include "Main.h"
#include "HDR.h"

#define SHRINK 0.25f
#define SAMPLE_COUNT 15

LPDIRECT3DTEXTURE9 CHDR::HDRTexture=NULL;
LPDIRECT3DTEXTURE9 CHDR::BrightTexture=NULL;
LPDIRECT3DTEXTURE9 CHDR::BloomTexture[BLOOM_COUNT];
bool CHDR::UseIntTexture=false;

//==============================================================
// HDR

// RXgN^
CHDR::CHDR(
	CMain* main, string title, 
	int mode, float tone_scale, 
	string mesh, D3DXVECTOR3 lpos, D3DXVECTOR3 lcol
)
:	CDemo(main, title), Yaw(0),
	Mode(mode), ToneScale(tone_scale), 
	LightPos(lpos), LightColor(D3DXVECTOR4(lcol, 1))
{
	// bṼ[h
	Mesh=LoadMesh(mesh);
	
	// GtFNg̃[h
	Effect=LoadEffect("HDR.fx");
}

// foCXZbg
void CHDR::OnResetDevice() {
	CDemo::OnResetDevice();

	// HDReNX`̍쐬
	float w=Graphics->GetWidth(), h=Graphics->GetHeight();
	if (!HDRTexture) {
	
		// eNX`̍쐬
		UseIntTexture=false;
		Device->CreateTexture(
			w, h, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A16B16G16R16F, 
			D3DPOOL_DEFAULT, &HDRTexture, NULL);

		// eNX`gpłȂꍇ̏
		if (!HDRTexture) {
			UseIntTexture=true;
			Device->CreateTexture(
				w, h, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A16B16G16R16, 
				D3DPOOL_DEFAULT, &HDRTexture, NULL);
			if (!HDRTexture) {
				Device->CreateTexture(
					w, h, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, 
					D3DPOOL_DEFAULT, &HDRTexture, NULL);
			}
		}

		// PxeNX`̍쐬
		Device->CreateTexture(
			w*SHRINK, h*SHRINK, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, 
			D3DPOOL_DEFAULT, &BrightTexture, NULL);

		// u[eNX`̍쐬
		for (int i=0; i<BLOOM_COUNT; i++) {
			BloomTexture[i]=NULL;
			Device->CreateTexture(
				w*SHRINK, h*SHRINK, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, 
				D3DPOOL_DEFAULT, &BloomTexture[i], NULL);
		}
	}
}
	
// foCXXg
void CHDR::OnLostDevice() {
	CDemo::OnLostDevice();
	SAFE_RELEASE(HDRTexture);
	SAFE_RELEASE(BrightTexture);
	for (int i=0; i<BLOOM_COUNT; i++) SAFE_RELEASE(BloomTexture[i]);
}

// fXgN^
CHDR::~CHDR() {
	OnLostDevice();
}
	
// ړ
void CHDR::Move() {
	Yaw+=0.01f;
}

// `
void CHDR::Draw() {

	// G[`FbN
	if (!PS20()) { Error="PS2.0"; return; }
	if (!Mesh) { Error="Mesh"; return; }
	if (!Effect) { Error="Effect"; return; }
	if (!HDRTexture) { Error="HDRTexture"; return; }
	if (!BrightTexture) { Error="BrightTexture"; return; }
	for (int i=0; i<BLOOM_COUNT; i++) {
		if (!BloomTexture[i]) { Error="BloomTexture"+ToStr(i); return; }
	}

	// [hs
	D3DXMATRIX world;
	D3DXMatrixRotationY(&world, Yaw);
	Effect->SetMatrix("World", &world);

	// [hr[s
	D3DXMATRIX w_view;
	D3DXVECTOR3 from(0, 3, -8);
	D3DXVECTOR3 lookat(0, 1, 0);
	D3DXVECTOR3 up(0, 1, 0);
	D3DXMatrixLookAtLH(&w_view, &from, &lookat, &up);
	D3DXMatrixMultiply(&w_view, &world, &w_view);
	Effect->SetMatrix("WView", &w_view);
	
	// [hr[ˉes
	D3DXMATRIX wv_proj;
	float w=Graphics->GetWidth(), h=Graphics->GetHeight();
	D3DXMatrixPerspectiveLH(&wv_proj, 1, h/w, 1, 1000);
	Effect->SetMatrix("Proj", &wv_proj);
	D3DXMatrixMultiply(&wv_proj, &w_view, &wv_proj);
	Effect->SetMatrix("WVProj", &wv_proj);

	// 
	D3DXVECTOR4 ambient(0.2f, 0.2f, 0.2f, 1);
	Effect->SetVector("Ambient", &ambient);

	// Cg
	D3DXVECTOR4 lpos(LightPos, 1);
	D3DXVec4Transform(&lpos, &lpos, &w_view);
	Effect->SetVector("LightVPos", &lpos);
	Effect->SetVector("LightColor", &LightColor);

	// p[^ƃeNjbN
	Effect->SetTechnique("HDRScene");
	Effect->SetFloat("IntScale", 1.00f);
	if (UseIntTexture) Effect->SetFloat("IntScale", 0.01f);

	// HDReNX`ɃV[`
	SetRenderTarget(HDRTexture, OrgDepthStencil);
	Graphics->Clear(BGColor);
	DrawMesh(Mesh, Effect);
	
	// Px̒o
	SetRenderTarget(BrightTexture, OrgDepthStencil);
	Graphics->Clear();
	Effect->SetTechnique("Bright");
	Effect->SetTexture("InputTex", HDRTexture);
	Effect->SetFloat("MinBright", 5.0f);
	DrawRect(Effect, 0, 0, w*SHRINK, h*SHRINK, 0, 0, 1, 1, D3DXCOLOR(1, 1, 1, 1));

	// u[̍쐬
	Effect->SetTechnique("Bloom");
	for (int j=0; j<BLOOM_COUNT; j++) {
		
		// TvÖʒuItZbgƃEFCgvZ
		D3DXVECTOR4 offset[SAMPLE_COUNT];
		float weight[SAMPLE_COUNT];
		float total=0;
		for (int i=0; i<SAMPLE_COUNT; i++) {
			float p=(i-(SAMPLE_COUNT-1)*0.5f)*0.01f;
			offset[i]=(j%2==0)?D3DXVECTOR4(p, 0, 0, 0):D3DXVECTOR4(0, p, 0, 0);
			weight[i]=exp(-p*p/2)/sqrt(D3DX_PI*2);
			total+=weight[i];
		}
		for (int i=0; i<SAMPLE_COUNT; i++) weight[i]/=total;

		// p[^̐ݒ		
		Effect->SetVectorArray("SampleOffset", offset, SAMPLE_COUNT);
		Effect->SetFloatArray("SampleWeight", weight, SAMPLE_COUNT);
		Effect->SetTexture("InputTex", j==0?BrightTexture:BloomTexture[j-1]);

		// eNX`Ƀu[`悷
		SetRenderTarget(BloomTexture[j], OrgDepthStencil);
		Graphics->Clear();
		DrawRect(Effect, 0, 0, w*SHRINK, h*SHRINK, 0, 0, 1, 1, D3DXCOLOR(1, 1, 1, 1));
	}

	// ŏIIȏo
	if (Mode==1) {
		SetRenderTarget(BloomTexture[BLOOM_COUNT-1], OrgDepthStencil);
		Graphics->Clear();
	}
	if (Mode==0 || Mode==1) {
		ResetRenderTarget();
		Effect->SetTechnique("Finish");
		Effect->SetFloat("ToneScale", ToneScale);
		Effect->SetTexture("InputTex", HDRTexture);
		Effect->SetTexture("BloomTex", BloomTexture[BLOOM_COUNT-1]);
		DrawRect(Effect, 0, 0, w, h, 0, 0, 1, 1, D3DXCOLOR(1, 1, 1, 1));
	}
	if (Mode==2) {
		ResetRenderTarget();
		DrawTexture(BrightTexture, 0, 0, w, h, 0, 0, 1, 1, D3DXCOLOR(1, 1, 1, 1));
	}
	if (Mode==3) {
		ResetRenderTarget();
		DrawTexture(BloomTexture[BLOOM_COUNT-1], 0, 0, w, h, 0, 0, 1, 1, D3DXCOLOR(1, 1, 1, 1));
	}
}

