#include "Main.h"
#include "MovieFilm.h"


//==============================================================
// ftB

// RXgN^
CMovieFilm::CMovieFilm(
	CMain* main, string title, int mode, 
	float i, float q, 
	float vibration, float flicker, float dust_rate, 
	float gx, float gy, 
	float min_value, float max_value
)
:	CDemo(main, title), Yaw(0), 
	NoiseShader(NULL), DustOrDummyTexture(NULL), 
	Mode(mode), I(i), Q(q), 
	Vibration(vibration), Flicker(flicker), DustRate(dust_rate), 
	GranularityX(gx), GranularityY(gy), 
	MinValue(min_value), MaxValue(max_value)
{
	// bṼ[h
	Mesh=LoadMesh("SceneDiningB.x");

	// GtFNg̃[h
	Effect=LoadEffect("MovieFilm.fx");
	
	// eNX`̃[h
	DustTexture=LoadTexture("Dust.png");

	// eNX`VF[_̃RpC
	LPD3DXBUFFER error;
	if (FAILED(D3DXCompileShaderFromFile(
		(ExePath+"..\\Effect\\MovieFilm.hlsl").c_str(), NULL, NULL, 
		"MovieFilmTX", "tx_1_0", D3DXSHADER_DEBUG,  
		&NoiseShaderBuffer, &error, NULL)) ||
		FAILED(D3DXCreateTextureShader(
			(const DWORD*)NoiseShaderBuffer->GetBufferPointer(), 
			&NoiseShader))
	) {
		// RpCG[̏ꍇ̓G[\ďI
		MessageBox(
			NULL, (LPCTSTR)error->GetBufferPointer(), 
			"Shader Compile Error", MB_OK);
		PostQuitMessage(1);
	}
}

// fXgN^
CMovieFilm::~CMovieFilm() {

	// eNX`VF[_̔j
	SAFE_RELEASE(NoiseShaderBuffer);
	SAFE_RELEASE(NoiseShader);

	// eNX`̔j
	SAFE_RELEASE(NoiseTexture);
}
	
// foCXXg̏
void CMovieFilm::OnLostDevice() {

	// mCYeNX`̔j
	SAFE_RELEASE(NoiseTexture);
}

// foCXZbg̏
void CMovieFilm::OnResetDevice() {
	CDemo::OnResetDevice();

	// mCYeNX`̍쐬
	LPDIRECT3DDEVICE9 device=Graphics->GetDevice();
	float w=Graphics->GetWidth(), h=Graphics->GetHeight();
	device->CreateTexture(
		w, h, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, 
		&NoiseTexture, NULL);
	if (NoiseTexture) {
		LPD3DXTEXTURESHADER ns=NoiseShader;
		if (ns) {
			ns->SetFloat(ns->GetConstantByName(NULL, "GranularityX"), GranularityX);
			ns->SetFloat(ns->GetConstantByName(NULL, "GranularityY"), GranularityY);
			ns->SetFloat(ns->GetConstantByName(NULL, "MinValue"), MinValue);
			ns->SetFloat(ns->GetConstantByName(NULL, "MaxValue"), MaxValue);
			D3DXFillTextureTX(NoiseTexture, NoiseShader);
		}
	}
}

// ړ
void CMovieFilm::Move() {
	Yaw+=0.01f;
	DrawX=Main->Rand05()*Vibration;
	DrawY=Main->Rand05()*Vibration;
	NoiseU=Main->Rand1();
	NoiseV=Main->Rand1();
	DustU=Main->Rand1();
	DustV=Main->Rand1();
	Brightness=1-Main->Rand1()*Flicker;
	DustOrDummyTexture=Main->Rand1()<DustRate?
		DustTexture:Main->GetDummyTexture();
}

// `
void CMovieFilm::Draw() {

	// G[`FbN
	if (!Mesh) { Error="Mesh"; return; }
	if (!Effect) { Error="Effect"; return; }
	if (!DustTexture) { Error="DustTexture"; return; }
	if (!NoiseShader) { Error="NoiseShader"; return; }
	if (!NoiseTexture) { Error="NoiseTexture"; return; }
	if (!PS20()) { Error="PS2.0"; return; }

	// [hs
	D3DXMATRIX world;
	D3DXMatrixRotationY(&world, Yaw);
	Device->SetTransform(D3DTS_WORLD, &world);

	// r[s
	D3DXMATRIX view;
	D3DXVECTOR3 from(0, 4, 8);
	D3DXVECTOR3 lookat(0, 1, 0);
	D3DXVECTOR3 up(0, 1, 0);
	D3DXMatrixLookAtLH(&view, &from, &lookat, &up);
	Device->SetTransform(D3DTS_VIEW, &view);
	
	// ˉes
	float w=Graphics->GetWidth(), h=Graphics->GetHeight();
	D3DXMATRIX proj;
	D3DXMatrixPerspectiveLH(&proj, 1, h/w, 1, 1000);
	Device->SetTransform(D3DTS_PROJECTION, &proj);

	// Cg
	Graphics->SetLight(0, D3DLIGHT_DIRECTIONAL, 1, -1, 1, 1000);

	// 
	D3DXCOLOR ambient(0.2f, 0.2f, 0.2f, 1);
	Device->SetRenderState(D3DRS_AMBIENT, ambient);

	// V[eNX`ɕ`悷
	SetRenderTarget(WorkTexture[0], WorkDepthStencil[0]);
	Graphics->Clear(BGColor);
	Mesh->Draw();

	// eNX`ʂɕ`悷
	ResetRenderTarget();
	Graphics->Clear();
	if (Mode==0) {
		Effect->SetFloat("I", I);
		Effect->SetFloat("Q", Q);
		Effect->SetFloat("NoiseU", NoiseU);
		Effect->SetFloat("NoiseV", NoiseV);
		Effect->SetFloat("DustU", DustU);
		Effect->SetFloat("DustV", DustV);
		Effect->SetTexture("MeshTex", WorkTexture[0]);
		Effect->SetTexture("NoiseTex", NoiseTexture);
		Effect->SetTexture("DustTex", DustOrDummyTexture);
		Effect->SetTechnique("MovieFilm");
		DrawRect(
			Effect, DrawX, DrawY, DrawX+w, DrawY+h, 0, 0, 1, 1, 
			D3DXCOLOR(Brightness, Brightness, Brightness, 1));
	} else 
	if (Mode==1) {
		DrawTexture(NoiseTexture, 0, 0, w, h, 0, 0, 1, 1, D3DXCOLOR(1, 1, 1, 1));
	}
}

