#include "Main.h"
#include "Reflection.h"


#define TEX_SIZE 512


//==============================================================
// 

// RXgN^
CReflection::CReflection(CMain* main, string title, float reflectivity)
:	CDemo(main, title), CubeTexture(NULL), 
	Yaw(0), Reflectivity(reflectivity)
{
	// bṼ[h
	MeshObj=LoadMesh("SceneHorror2_Obj.x");
	MeshEnv=LoadMesh("SceneHorror2_Env.x");

	// GtFNg̃[h
	Effect=LoadEffect("Reflection.fx");
}

// foCXXg̏
void CReflection::OnLostDevice() {
	CDemo::OnLostDevice();
	
	SAFE_RELEASE(CubeTexture);
	SAFE_RELEASE(DepthStencil);
}

// foCXZbg̏
void CReflection::OnResetDevice() {
	CDemo::OnResetDevice();

	// eNX`̍쐬
	Device->CreateCubeTexture(
		TEX_SIZE, 1, D3DUSAGE_RENDERTARGET,
		D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &CubeTexture, NULL);

	// [xobt@̍쐬
	Device->CreateDepthStencilSurface(
		TEX_SIZE, TEX_SIZE, D3DFMT_D16, 
		D3DMULTISAMPLE_NONE, 0, TRUE, &DepthStencil, NULL);
}

// fXgN^
CReflection::~CReflection() {
	OnLostDevice();
}
	
// ړ
void CReflection::Move() {
	Yaw+=0.01f;
}

// `iL[ueNX`j
void CReflection::DrawCubeTexture() {

	// [hs
	D3DXMATRIX world;
	D3DXMatrixRotationY(&world, Yaw);
	Effect->SetMatrix("World", &world);

	// Cg
	D3DXVECTOR4 light(1, -1, 1, 0);
	D3DXVec4Normalize(&light, &light);
	Effect->SetVector("Light", &light);

	// 
	D3DXVECTOR4 ambient(0.2f, 0.2f, 0.2f, 1);
	Effect->SetVector("Ambient", &ambient);

	// eNjbN
	Effect->SetTechnique("Basic");

	// 6ʕ̕`
	for (int face=0; face<6; face++) {
	
		// r[svZp̃xNg
		static const D3DXVECTOR3 lookat[]={
			D3DXVECTOR3(1, 0, 0), D3DXVECTOR3(-1, 0, 0), 
			D3DXVECTOR3(0, 1, 0), D3DXVECTOR3(0, -1, 0), 
			D3DXVECTOR3(0, 0, 1), D3DXVECTOR3(0, 0, -1)
		};
		static const D3DXVECTOR3 up[]={
			D3DXVECTOR3(0, 1, 0), D3DXVECTOR3(0, 1, 0), 
			D3DXVECTOR3(0, 0, -1), D3DXVECTOR3(0, 0, 1), 
			D3DXVECTOR3(0, 1, 0), D3DXVECTOR3(0, 1, 0)
		};

		// [hr[s
		D3DXMATRIX w_view;
		D3DXVECTOR3 from(0, 1.1f, -1.0f);
		D3DXVECTOR3 la=from+lookat[face];
		D3DXMatrixLookAtLH(&w_view, &from, &la, &up[face]);
		D3DXMatrixMultiply(&w_view, &world, &w_view);
		Effect->SetMatrix("WView", &w_view);
		
		// [hr[ˉes
		D3DXMATRIX wv_proj;
		D3DXMatrixPerspectiveLH(&wv_proj, 1, 1, 0.5f, 1000);
		D3DXMatrixMultiply(&wv_proj, &w_view, &wv_proj);
		Effect->SetMatrix("WVProj", &wv_proj);

		// _O^[QbgieNX`j
		SetRenderTarget(CubeTexture, face, DepthStencil);
		Graphics->Clear(BGColor);

		// bViwij̕`
		DrawMesh(MeshEnv, Effect);
	}
}

// `
void CReflection::Draw() {

	// G[`FbN
	if (!MeshObj) { Error="MeshObj"; return; }
	if (!MeshEnv) { Error="MeshEnv"; return; }
	if (!Effect) { Error="Effect"; return; }

	// `iL[ueNX`j
	DrawCubeTexture();

	// [hs
	D3DXMATRIX world;
	D3DXMatrixRotationY(&world, Yaw);
	Effect->SetMatrix("World", &world);

	// [hr[s
	D3DXMATRIX w_view;
	D3DXVECTOR3 from(0, 3, -8);
	D3DXVECTOR3 lookat(0, 1, 0);
	D3DXVECTOR3 up(0, 1, 0);
	D3DXMatrixLookAtLH(&w_view, &from, &lookat, &up);
	D3DXMatrixMultiply(&w_view, &world, &w_view);
	Effect->SetMatrix("WView", &w_view);
	
	// [hr[ˉes
	D3DXMATRIX wv_proj;
	float w=Graphics->GetWidth(), h=Graphics->GetHeight();
	D3DXMatrixPerspectiveLH(&wv_proj, 1, h/w, 1, 1000);
	D3DXMatrixMultiply(&wv_proj, &w_view, &wv_proj);
	Effect->SetMatrix("WVProj", &wv_proj);

	// Cg
	D3DXVECTOR4 light(1, -1, 1, 0);
	D3DXVec4Normalize(&light, &light);
	Effect->SetVector("Light", &light);

	// 
	D3DXVECTOR4 ambient(0.2f, 0.2f, 0.2f, 1);
	Effect->SetVector("Ambient", &ambient);

	// _O^[Qbgiʁj
	ResetRenderTarget();
	Graphics->Clear();

	// bViwij̕`
	Effect->SetTechnique("Basic");
	DrawMesh(MeshEnv, Effect);

	// bViIuWFNgj̕`
	Effect->SetTexture("EnvTex", CubeTexture);
	Effect->SetFloat("Reflectivity", Reflectivity);
	Effect->SetTechnique("Reflection");
	DrawMesh(MeshObj, Effect);
}

